'''
% Kolmogorov-Smirnov Test
% This program conducts Kolmogorov-Smirnov test with survival data.
%
% This program requires R, rpy, and surv2sample (R package).
%
%    R ==> http://www.r-project.org/
%    rpy ==> http://rpy.sourceforge.net/
%    surv2sample (R package) ==> http://cran.r-project.org/web/packages/surv2sample/index.html
%
% Last Update: 2011.06.03
% By Jae-Seong Yang, destine@postech.ac.kr
'''

from rpy import *
import SurvivalData
import os, sys


def InitR():
    r.library( "surv2sample" )

def KSTest( time, event, treatment ):
    r.assign( 'tt', time )
    r.assign( 'ee', event )
    r.assign( 'tr', treatment )

    result = r("surv2.ks( Surv( tt, ee), tr)") 

    return result

def KSTestAll( conditions ):
    results_ksTest = []

    for i in range( 0, len(conditions) ):
        aData1 = conditions[i]
        ( time1, event1 ) = aData1.GetSurvData() 

        for j in range( 0, len(conditions) ):
            if i == j:
                continue
            aData2 = conditions[j]

            ( time2, event2 ) = aData2.GetSurvData() 

            treatment2 = [ 2 for x in time2 ]
            treatment2.extend( [ 1 for y in time1 ] )

            time2.extend( time1 )
            event2.extend( event1 )

            result = KSTest( time2, event2, treatment2 )

            Statistics = result['stat.ks']
            P_value = result['pval.ks']
            Corrected_P_value = min( P_value * ( len( conditions ) - 1 ), 1.0 )            

            results_ksTest.append( [ aData1.name, aData2.name, Statistics, P_value, Corrected_P_value ] )

    return results_ksTest
    
if __name__ == "__main__":
    InitR()

    conditions = SurvivalData.ReadDataFromFile( "sample.txt" )

    results_ksTest = KSTestAll( conditions )

    print "\t".join( [ "Sample 1", "Sample 2", "Statistics", "P-value", "Corrected P-value" ] )
    for results in results_ksTest:
        for v in results:
            print v, "\t",
        print ""
